@@ -9,11 +9,10 @@
 #' @importFrom dplyr tbl_vars
 #' @importFrom utils type.convert
 #' @importFrom lifecycle deprecated
-#' @importFrom tidyselect any_of
 #' @useDynLib tidyr, .registration = TRUE
 "_PACKAGE"
 
-globalVariables(".")
+globalVariables(c(".", "name", "value"))
 
 #' @importFrom tibble tribble
 #' @export
@@ -29,6 +28,12 @@ tibble::as_tibble
 
 
 #' @aliases select_helpers
+#' @importFrom tidyselect all_of
+#' @export
+tidyselect::all_of
+#' @importFrom tidyselect any_of
+#' @export
+tidyselect::any_of
 #' @importFrom tidyselect contains
 #' @export
 tidyselect::contains
@@ -38,6 +43,9 @@ tidyselect::ends_with
 #' @importFrom tidyselect everything
 #' @export
 tidyselect::everything
+#' @importFrom tidyselect last_col
+#' @export
+tidyselect::last_col
 #' @importFrom tidyselect matches
 #' @export
 tidyselect::matches
@@ -50,6 +58,3 @@ tidyselect::one_of
 #' @importFrom tidyselect starts_with
 #' @export
 tidyselect::starts_with
-#' @importFrom tidyselect last_col
-#' @export
-tidyselect::last_col
