@@ -1,5 +1,3 @@
-context("Separate")
-
 test_that("missing values in input are missing in output", {
   df <- tibble(x = c(NA, "a b"))
   out <- separate(df, x, c("x", "y"))
@@ -51,7 +49,7 @@ test_that("convert keeps characters as character", {
 test_that("too many pieces dealt with as requested", {
   df <- tibble(x = c("a b", "a b c"))
 
-  expect_warning(separate(df, x, c("x", "y")), "Additional pieces discarded")
+  expect_snapshot(separate(df, x, c("x", "y")))
 
   merge <- separate(df, x, c("x", "y"), extra = "merge")
   expect_equal(merge[[1]], c("a", "a"))
@@ -61,13 +59,13 @@ test_that("too many pieces dealt with as requested", {
   expect_equal(drop[[1]], c("a", "a"))
   expect_equal(drop[[2]], c("b", "b"))
 
-  expect_warning(separate(df, x, c("x", "y"), extra = "error"), "deprecated")
+  expect_snapshot(separate(df, x, c("x", "y"), extra = "error"))
 })
 
 test_that("too few pieces dealt with as requested", {
   df <- tibble(x = c("a b", "a b c"))
 
-  expect_warning(separate(df, x, c("x", "y", "z")), "Missing pieces filled")
+  expect_snapshot(separate(df, x, c("x", "y", "z")))
 
   left <- separate(df, x, c("x", "y", "z"), fill = "left")
   expect_equal(left$x, c(NA, "a"))
@@ -82,14 +80,14 @@ test_that("preserves grouping", {
   df <- tibble(g = 1, x = "a:b") %>% dplyr::group_by(g)
   rs <- df %>% separate(x, c("a", "b"))
   expect_equal(class(df), class(rs))
-  expect_equal(dplyr::groups(df), dplyr::groups(rs))
+  expect_equal(dplyr::group_vars(df), dplyr::group_vars(rs))
 })
 
 test_that("drops grouping when needed", {
   df <- tibble(x = "a:b") %>% dplyr::group_by(x)
   rs <- df %>% separate(x, c("a", "b"))
   expect_equal(rs$a, "a")
-  expect_equal(dplyr::groups(rs), NULL)
+  expect_equal(dplyr::group_vars(rs), character())
 })
 
 test_that("overwrites existing columns", {
@@ -109,14 +107,36 @@ test_that("drops NA columns", {
 
 test_that("checks type of `into` and `sep`", {
   df <- tibble(x = "a:b")
-  expect_error(
-    separate(df, x, "x", FALSE),
-    "must be either numeric or character"
-  )
-  expect_error(
-    separate(df, x, FALSE),
-    "must be a character vector"
-  )
+
+  expect_snapshot({
+    (expect_error(separate(df, x, "x", FALSE)))
+    (expect_error(separate(df, x, FALSE)))
+  })
+})
+
+test_that("informative error if using stringr modifier functions (#693)", {
+  df <- tibble(x = "a")
+  sep <- structure("a", class = "pattern")
+
+  expect_snapshot((expect_error(separate(df, x, sep = sep))))
+})
+
+# helpers -----------------------------------------------------------------
+
+test_that("str_split_n can cap number of splits", {
+  expect_equal(str_split_n(c("x,x"), ",", 1), list("x,x"))
+  expect_equal(str_split_n(c("x,x"), ",", 2), list(c("x", "x")))
+  expect_equal(str_split_n(c("x,x"), ",", 3), list(c("x", "x")))
+})
+
+test_that("str_split_n handles edge cases", {
+  expect_equal(str_split_n(character(), ",", 1), list())
+  expect_equal(str_split_n(NA, ",", 1), list(NA_character_))
+})
+
+test_that("str_split_n handles factors", {
+  expect_equal(str_split_n(factor(), ",", 1), list())
+  expect_equal(str_split_n(factor("x,x"), ",", 2), list(c("x", "x")))
 })
 
 test_that("list_indices truncates long warnings", {
